#! /bin/sh

### Lock ######################################################################
#                                                                             #
#  Usage : lock.sh [-n group name of lock (default:"lock") ] \                #
#          [-t seconds until ignoring lock file (default:"60") | -c | -u ]    #
#                                                                             #
#  (Ret) =0 ... Locking or Unlocking or Continuing has been succeed.          #
#        >0 ... Locking or Unlocking or Continuing has been failed.           #
#                                                                             #
###############################################################################


# =============================================================================
# Definition
# =============================================================================
#
# ----- define common variables and functions -----
. /usr/local/richscripts/etc/common.lib


# =============================================================================
# Parse the arguments
# -----------------------------------------------------------------------------
#  1. parse the name of lock
#  2. parse the ignoring time
#  3. set default values if no values were set
# =============================================================================
#
# ----- 1. 2. parse lock name and ignoring time -----
arg_switch=""
for arg in $*
do
  if [ "$arg_switch" = "" ]; then
    if   [ "$arg" = "-c" ]; then
      action="CONTINUE"
    elif [ "$arg" = "-u" ]; then
      action="UNLOCK"
    else
      arg_switch="$arg"
    fi
  elif [ "$arg_switch" = "-n" ]; then
    arg_switch=""
    lock_name="$arg"
  elif [ "$arg_switch" = "-t" ]; then
    arg_switch=""
    check=`echo $arg | $CMD_GREP -E ^[0-9]*$`
    if [ -n "$check" ]; then
      ignoring_time="${arg}"
    else
      check=`echo $arg | $CMD_GREP -E ^[0-9]*[smhdw]$`
      if [ -n "$check" ]; then
        case ${check##*[0-9]} in
          s)
            ignoring_time=${check%s}
            ;;
          m)
            ignoring_time=`expr ${check%m} \* 60`
            ;;
          h)
            ignoring_time=`expr ${check%h} \* 3600`
            ;;
          d)
            ignoring_time=`expr ${check%d} \* 86400`
            ;;
          w)
            ignoring_time=`expr ${check%w} \* 604800`
            ;;
        esac
      fi
    fi
  fi
done
# ----- 3. set default values if no values were set -----
[ -z "$action" ] && action="LOCK"
[ -z "$lock_name" ] && lock_name="lock"
[ -z "$ignoring_time" ] && ignoring_time="60"


# =============================================================================
# Main
# =============================================================================
#
if   [ "$action" = "LOCK" ]; then
  locking="NOTYET"
  while [ "$locking" != "DONE" ]; do
    if [ "$locking" != "RETRY" ]; then
      $CMD_MKDIR "${PATH_LOCK}/${lock_name}" > /dev/null 2>&1
    else
      $CMD_RMDIR "${PATH_LOCK}/${lock_name}"
      $CMD_MKDIR "${PATH_LOCK}/${lock_name}" > /dev/null 2>&1
    fi
    ret="$?"
    if [ "$locking" != "RETRY" ]; then
      if   [ "$ret" -eq 0 ]; then
        locking="DONE"              # succeeded to lock
      elif [ "$ret" -eq 1 ]; then
        case $OS_TYPE in
          FreeBSD)
            check=`$CMD_FIND "${PATH_LOCK}/" -name "${lock_name}" \
                   -mtime +${ignoring_time}s`
            if [ -n "$check" ]; then
              locking="RETRY"       # failed to lock but lock file is too old
            else
              locking="DONE"        # failed to lock and lock file
                                    # is in the term of validity
            fi
            ;;
          Linux)
            path_tmpfile=`$CMD_MKTEMP "${PATH_LOCK}/tmpfile.XXXXXXXX"`
            if [ $? -eq 0 ]; then
              check=`$CMD_FIND $path_tmpfile -printf "%T@"`
              check=`expr $check - $ignoring_time`
              rm -f $path_tmpfile
              if [ "`$CMD_FIND "${PATH_LOCK}/${lock_name}" -printf "%T@"`" \
                   -gt "$check" ]; then
                locking="RETRY"     # failed to lock but lock file is too old
              else
                locking="DONE"      # failed to lock and lock file
                                    # is in the term of validity
              fi
            else
              locking="DONE"        # failed to lock and check expiry
            fi
            ;;
        esac
      else
        locking="DONE"              # failure the locking operation
      fi
    else
      locking="DONE"                # unexpected
    fi
  done
elif [ "$action" = "UNLOCK" ]; then
  if [ -d "${PATH_LOCK}/${lock_name}" ]; then
    $CMD_RMDIR "${PATH_LOCK}/${lock_name}"
    ret="$?"
  else
    ret="0"
  fi
elif [ "$action" = "CONTINUE" ]; then
  if [ -d "${PATH_LOCK}/${lock_name}" ]; then
    $CMD_TOUCH "${PATH_LOCK}/${lock_name}"
    ret="$?"
  else
    ret="1"
  fi
fi
exit $ret
