#! /bin/sh

### DNS Forwarding Serevers Registring Script for djbdns dnscache #############
#                                                                             #
#  Usage : regdns.sh -f file1 [file2 ...] \                                   #
#          { -n nameserver1 [nameserver2 ...] | -ofix | -ffix }               #
#                                                                             #
#  (file)      : forwarding domain file (Do not contain "/" in filename)      #
#  (nameserver): forwarding nameserver specified with "x.x.x.x" style         #
#  "-ofix"     : load nameservers list written in the configuration file      #
#                ("ons1" and "ons2") as fixed nameservers for ISP             #
#  "-ffix"     : load nameservers list written in the configuration file      #
#                ("fns1" and "fns2") as fixed nameservers for Flet's Square   #
#  If you want to delele the forwarding domain files, specify "DELETE" as     #
#  a nameserver.                                                              #
#  "-n" (nameserver) and "-ofix" and "-ffix" are exclusive each other.        #
#                                                                             #
###############################################################################


# =============================================================================
# Definition
# =============================================================================
#
# ----- define common variables and functions -----
. /usr/local/richscripts/etc/common.lib


# =============================================================================
# Exit if not required
# =============================================================================
#
[ -z "$CMD_SVC" ] && exit


# =============================================================================
# Parse the arguments
# -----------------------------------------------------------------------------
#  1. parse nameservers
#  2. parse output files
#  3. load fixed nameservers list from the configuration file if need
#  4. show help message and exit if not enough argument
# =============================================================================
#
# ----- 1. 2. parse nameservers and output files -----
outputfiles=""
nameservers=""
arg_mode=""
for arg in $*
do
  case "$arg" in
    -ofix)
        arg_mode=""
        fixed_ns="ISP"
        ;;
    -ffix)
        arg_mode=""
        fixed_ns="FLETS"
        ;;
    -*) # --- Parse option switches
        for mode in "-n" "-f"
        do
          if [ "$arg" = "$mode" ]; then
            arg_mode="$arg"
            break
          fi
        done
        ;;
    *)  # --- Parse the parameters
        if [ "$arg_mode" = "-n" ]; then
          fixed_ns=""
          # --- Register in nameserver list only when valid IP address
          check=`echo $arg | $CMD_GREP -E $PATTERN_IPV4`
          if [ -n "$check" ]; then
            if [ -z "$nameservers" ]; then
              nameservers="$arg"
            else
              nameservers="$nameservers $arg"
            fi
          else
            case "$arg" in
              [Dd][Ee][Ll][Ee][Tt][Ee])
                nameservers="DELETE"
                ;;
            esac
          fi
        elif [ "$arg_mode" = "-f" ]; then
          # --- Register in output file list only when pure filename
          check=`echo $arg | $CMD_GREP -v /`
          if [ -n "$check" ]; then
            if [ -n "$outputfiles" ]; then
              outputfiles="$outputfiles $arg"
            else
              outputfiles="$arg"
            fi
          fi
          case "$arg" in
            /*)
                if [ -n "$outputfiles" ]; then
                  outputfiles="$outputfiles $arg"
                else
                  outputfiles="$arg"
                fi
                ;;
          esac
        fi
        ;;
  esac
done
# ----- 3. load fixed nameservers list from the configuration file if need -----
if [ -n "$fixed_ns" ]; then
  nameservers=""
  if   [ "$fixed_ns" = "ISP" ]; then
    nameservers0="$ons1 $ons2"
  elif [ "$fixed_ns" = "FLETS" ]; then
    nameservers0="$fns1 $fns2"
  fi
  for nameserver in $nameservers0
  do
    check=`echo $nameserver | $CMD_GREP -E $PATTERN_IPV4`
    if [ -n "$check" ]; then
      if [ -z "$nameservers" ]; then
        nameservers="$nameserver"
      else
        nameservers="$nameservers $nameserver"
      fi
    fi
  done
fi
# ----- 4. show help message and exit if the arguments are not enough -----
if [ \( -z "$outputfiles" \) -o \( -z "$nameservers" \) ]; then
  echo "Usage : ${0##*/} -f file1 [file2 ...] \\"
  echo "        { -n nameserver1 [nameserver2 ...] | -ofix | -ffix }"
  echo
  echo "\"-n\"    nameserver must be the x.x.x.x style or \"DELETE\"."
  echo
  echo "\"-f\"    file must be pure filename. (Do not contain \"/\")"
  echo
  echo "\"-ofix\" load nameservers list written in the configuration file"
  echo "        (\"ons1\" and \"ons2\") as fixed nameservers for ISP"
  echo
  echo "\"-ffix\" load nameservers list written in the configuration file"
  echo "        (\"fns1\" and \"fns2\") as fixed nameservers for Flet's Square"
  echo
  echo "\"-n\" and \"-ofix\" and \"-ffix\" are exclusive each other."
  debug_log "regdns.sh: the arguments are not enough"
  exit
fi


# =============================================================================
# Check and update each nameserver list file
# -----------------------------------------------------------------------------
#  Compare current nameserver list and new list for checking whether need to
#  update or not at first. If need, update the list file and keep the registry
#  in the output file list. Otherwise, delete registry from the output file
#  list and discard the new list.
# =============================================================================
#
outputfiles0="$outputfiles"
outputfiles=""
for outputfile in $outputfiles0
do
  # --- define temporary files
  curfile="$DIR_DNSCACHE/root/servers/$outputfile"
  oldfile="$curfile.old"
  newfile="$curfile.new"
  if [ "${nameservers%% *}" != "DELETE" ]; then
    # --- save sorted old server list file
    if [ -f "$curfile" ]; then
      $CMD_SORT $curfile > $oldfile
    else
      echo -n > $oldfile
    fi
    # --- create the new server list file
    echo -n > $newfile
    for nameserver in $nameservers
    do
      echo $nameserver >>$newfile
    done
    # --- check whether the file is updated or not
    $CMD_SORT $newfile | $CMD_DIFF --brief - $oldfile > /dev/null
    if [ $? -eq 1 ]; then
      check="KEEP"
    else
      check="DISCARD"
    fi
  else
    if [ -f "$curfile" ]; then
      $CMD_RM -f $curfile
      check="KEEP"
    else
      check="DISCARD"
    fi
  fi
  # --- keep the file in list and overwrite current file when updated
  #     otherwise delete the new file
  if [ "$check" = "KEEP" ]; then
    if [ -n "$outputfiles" ]; then
      outputfiles="$outputfiles $outputfile"
    else
      outputfiles="$outputfile"
    fi
    [ -f "$newfile" ] && $CMD_MV -f $newfile $curfile
  else
    [ -f "$newfile" ] && $CMD_RM -f $newfile
  fi
  # --- the old server list file will be delete
  [ -f "$oldfile" ] && $CMD_RM -f $oldfile
done


# =============================================================================
# Reboot the dnscache if need
# -----------------------------------------------------------------------------
#  At first, check the following condensations
#  1. one or more files are updated (It is evident with checking whether
#     $outputfiles is NULL or not.)
#  2. the dnscache is running now
#
#  If both condensations are sutisfied,
#  3. Reboot the dnscache
# =============================================================================
#
# ----- 1. Check whether one or more files are updated -----
if [ -z "$outputfiles" ]; then
  exit
fi
# ----- 2. Check whether the dnscache is running now -----
check=`$CMD_PS ax | $CMD_GREP ${DIR_DNSCACHE##/*/} | $CMD_GREP -v ${CMD_GREP##/*/}`
if [ -z "$check" ]; then
  exit
fi
# ----- 3. Reboot the dnscache -----
$CMD_SVC -t $DIR_DNSCACHE $DIR_DNSCACHE/log
